/********************************************************
 * 	eBOOKLEARNING - 
 *	E-book-based interactive learning system
 * ======================================================
 *
 * License: http://www.gnu.org/licenses/gpl.html
 * @author Florence Adegeye
 * @version: 1.0, 2015/01/31
 *
 ********************************************************/
package at.tuwien.ebooklearning.ebookservices;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;

import nl.siegmann.epublib.domain.Book;
import nl.siegmann.epublib.domain.Resource;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.LevelListDrawable;
import android.text.Html.ImageGetter;
import android.util.Log;
import at.tuwien.ebooklearning.R;

public class BookImageLoader implements ImageGetter {

	 private Context context;
	 private Book book;
	 private HashMap<String, Bitmap> imageCache;

	public BookImageLoader(Context context, BookInfo current) {
		super();
		this.context = context;
		this.book = current.getEpub();
		this.imageCache = new HashMap<String, Bitmap>();
	
	}
	
	@Override
	public Drawable getDrawable(String source) {
		// TODO Auto-generated method stub
		LevelListDrawable drawableList = new LevelListDrawable();
		 Bitmap bitmap = null;
		 try {
	            
	            Resource resource = book.getResources().getByHref(source);
	            byte[] data = resource.getData();
	          
	            bitmap = BitmapFactory.decodeByteArray(data, 0, data.length);
	        	this.imageCache.put(source, bitmap);
	           
	        	
		  }catch (OutOfMemoryError e){
			  Log.e("OutOfMemoryError", "Could not load image " + source);
		  }catch (FileNotFoundException e) {
	            e.printStackTrace();
	      }catch (MalformedURLException e) {
	            e.printStackTrace();
	      }catch (IOException e) {
	            e.printStackTrace();
	      }
		 Bitmap cachedBitmap = null; 
		 if (!this.imageCache.isEmpty()) {
			cachedBitmap = this.imageCache.get(source);
		 }
		
		if (cachedBitmap != null) {
        	
            BitmapDrawable mDrawable = new BitmapDrawable(context.getResources(), cachedBitmap);
        	
            drawableList.addLevel(1, 1, mDrawable);
            drawableList.setBounds(0, 0,  cachedBitmap.getWidth(),  cachedBitmap.getHeight());
            drawableList.setLevel(1);
         
        }else{
        	  Drawable empty =  this.context.getResources().getDrawable(R.drawable.ic_launcher);
        	  
              drawableList.addLevel(0, 0, empty);
              drawableList.setBounds(0, 0, empty.getIntrinsicWidth(), empty.getIntrinsicHeight());
        }
        
		 return drawableList;
      
       

	
	}
	
}

