/********************************************************
 * 	eBOOKLEARNING - 
 *	E-book-based interactive learning system
 * ======================================================
 *
 * License: http://www.gnu.org/licenses/gpl.html
 * @author Florence Adegeye
 * @version: 1.0, 2015/01/31
 *
 ********************************************************/
package at.tuwien.ebooklearning.ebookservices;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;

import nl.siegmann.epublib.domain.Book;
import nl.siegmann.epublib.epub.EpubReader;
import android.util.Log;

public class BooksManager {
	private static HashMap<Integer, BookInfo> allBooks = new HashMap<Integer, BookInfo>();
	private static BookInfo currentBook;
	private static int bookID = 0;
	
	static {
	     loadbooks();
	}
	
	public static HashMap<Integer, BookInfo> loadbooks(){
		return allBooks;
		
	}
	
	public static void savebooks(HashMap<Integer, BookInfo> mybooks){
		allBooks = mybooks;
	}
	
	
/**
	 * @return the currentBook
	 */
	public static BookInfo getCurrentBook() {
		return currentBook;
	}

	/**
	 * @param currentBook the currentBook to set
	 */
	public static void setCurrentBook(BookInfo currentBook) {
		BooksManager.currentBook = currentBook;
	}

public static boolean loadEPubs(String path, String extension){
	
	
		Log.e("path", "path: " + path);
		File bookfiles = new File(path);
		
		Log.e("path", "f: " + bookfiles);
		if(!bookfiles.isDirectory()){
			Log.e("loadEPub", "Directory does not exist");
			return false;
		}else{
			
			File[] filelist =  bookfiles.listFiles();
			Log.i("file","filepath: "+ bookfiles.listFiles());
			if(filelist != null){
				if(filelist.length != 0){
					
					for (File file : filelist) {
						
						if(file.isFile() && file.getName().endsWith(extension)){
							BookInfo currentBookInfo = createBookInfo(file);
							currentBookInfo.setBookid(bookID);
							allBooks.put(bookID, currentBookInfo);	
							bookID++;
							
						}else if(file.isDirectory()){
							
							loadEPubs(file.getAbsolutePath(), extension);
						}
					}
					
					return true; 
				}else{
					//show text to import files
					
					Log.e("loadePubs", "No files imported");
					
				}	
			}else{
				Log.e("loadedPubs", "Filelist error");
			}
			
			return false;
			
		}
	
	}
	
public static BookInfo createBookInfo(File file){
	BookInfo myBook = new BookInfo();
	
	//create epub
	InputStream epubInputStream;
	try {
		epubInputStream = new BufferedInputStream(new FileInputStream(file));
		
		if(epubInputStream != null){
			 Book book = (new EpubReader()).readEpub(epubInputStream);
			 myBook.setEpub(book);
			 myBook.setTitle(book.getTitle());
			 myBook.setShortInfo(book.getTitle(), book.getMetadata().getAuthors());
			 myBook.setLanguage(book.getMetadata().getLanguage());
		
			 myBook.setBookfile(file);
			 myBook.setFilename(file.getName());
			 myBook.setFilepath(file.getAbsolutePath());
			 myBook.setFilesize(file.length());
		 }
		
	}catch (OutOfMemoryError e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	catch (FileNotFoundException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	} catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	} 
	 
	return myBook;
}
	
	

}
