/********************************************************
 * 	eBOOKLEARNING - 
 *	E-book-based interactive learning system
 * ======================================================
 *
 * License: http://www.gnu.org/licenses/gpl.html
 * @author Florence Adegeye
 * @version: 1.0, 2015/01/31
 *
 ********************************************************/
package at.tuwien.ebooklearning.tts;

import java.util.Locale;

import android.content.Context;
import android.media.AudioManager;
import android.speech.tts.TextToSpeech;
import android.speech.tts.TextToSpeech.OnInitListener;
import android.util.Log;


public class TTSHandler implements OnInitListener{
	private TTSLanguages textLang;
	private TextToSpeech TTSObj; 

	public TTSHandler(TTSLanguages textLang, Context context) {
		super();
		this.textLang = textLang;
		this.TTSObj = new TextToSpeech(context, this);
		//TTSObj.setSpeechRate(0.8f);
		AudioManager am = (AudioManager)context.getSystemService(Context.AUDIO_SERVICE);
		am.setStreamVolume(AudioManager.STREAM_MUSIC, 6, 0);
		
	}
	
	public void startSpeaking(String text){
		if(TTSObj != null){
			TTSObj.speak(text, TextToSpeech.QUEUE_FLUSH, null);
		}
		
	}

	public void stopSpeaking(){
		 if(TTSObj !=null){
			 TTSObj.stop();
			 TTSObj.shutdown();
	      }
	    
	}
	
	@Override
	public void onInit(int status) {
		// TODO Auto-generated method stub
		if(status != TextToSpeech.ERROR){
			if(this.textLang == TTSLanguages.EN){
				this.TTSObj.setLanguage(Locale.UK);	
				Log.i(TTSHandler.class.getName(),"English language");
			}else if(this.textLang == TTSLanguages.DE){
				this.TTSObj.setLanguage(Locale.GERMANY);
				Log.i(TTSHandler.class.getName(),"German language");
			}else{
				this.TTSObj.setLanguage(Locale.getDefault());
				Log.i(TTSHandler.class.getName(),"Default language");
			}
		}
	}

	/**
	 * @return the textLang
	 */
	public TTSLanguages getTextLang() {
		return textLang;
	}

	/**
	 * @param textLang the textLang to set
	 */
	public void setTextLang(TTSLanguages textLang) {	
		if(textLang == TTSLanguages.EN){
			if(TTSObj.isLanguageAvailable(Locale.UK) == TextToSpeech.LANG_COUNTRY_AVAILABLE){
				Log.i(TTSHandler.class.getName(),"English language");
				this.textLang = textLang;	
				this.TTSObj.setLanguage(Locale.UK);	
			}else{
				Log.i(TTSHandler.class.getName(),"Language not available");
				this.textLang = TTSLanguages.DEFAULT;
				this.TTSObj.setLanguage(Locale.getDefault());
			}
		
		}else if(textLang == TTSLanguages.DE){
			if(TTSObj.isLanguageAvailable(Locale.GERMANY) == TextToSpeech.LANG_COUNTRY_AVAILABLE){
				Log.i(TTSHandler.class.getName(),"German language");
				this.textLang = textLang;
				this.TTSObj.setLanguage(Locale.GERMANY);
				
			}else{
				Log.i(TTSHandler.class.getName(),"Language not available");
				this.textLang = TTSLanguages.DEFAULT;
				this.TTSObj.setLanguage(Locale.getDefault());
			}
			
		}else{
			this.textLang = TTSLanguages.DEFAULT;
			this.TTSObj.setLanguage(Locale.getDefault());
			Log.i(TTSHandler.class.getName(),"Default language");
		}
		
		
		
		
	}
	
	
	
}
