/********************************************************
 * 	eBOOKLEARNING - 
 *	E-book-based interactive learning system
 * ======================================================
 *
 * License: http://www.gnu.org/licenses/gpl.html
 * @author Florence Adegeye
 * @version: 1.0, 2015/01/31
 *
 ********************************************************/
package at.tuwien.ebooklearning.utils;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.Color;
import android.text.Html;
import android.text.SpannableStringBuilder;
import android.text.method.ScrollingMovementMethod;
import android.text.style.TextAppearanceSpan;
import android.util.DisplayMetrics;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.TextSwitcher;
import android.widget.TextView;
import android.widget.TextView.BufferType;
import android.widget.ViewSwitcher.ViewFactory;
import at.tuwien.ebooklearning.R;
import at.tuwien.ebooklearning.dictionary.DictionaryEntry;
import at.tuwien.ebooklearning.dictionary.DictionaryHandler;
import at.tuwien.ebooklearning.dictionary.DictionaryLanguages;
import at.tuwien.ebooklearning.dictionary.DictionaryWebServices;
import at.tuwien.ebooklearning.ebookservices.BookInfo;
import at.tuwien.ebooklearning.tts.TTSHandler;
import at.tuwien.ebooklearning.tts.TTSLanguages;
import at.tuwien.ebooklearning.views.ContentActivity;
import at.tuwien.ebooklearning.webservices.IDictionaryResult;

@SuppressLint("DefaultLocale")
public class LearningManager implements IDictionaryResult {
	
	private TextSwitcher learningView;
	private Button closeButton;
	private DictionaryHandler myDictionary;
	private TTSHandler myPronunciation;
	private LearningServices learningService;
	private Context context;
	private String word="";
	
	
	public LearningManager(final Context context, TextSwitcher textswitcher, Button closeButton, BookInfo currentBook) {
		
		this.context = context;
		
		
		//Add learning services
		if(currentBook != null){
			if(currentBook.getLanguage().equals("en")){
				myDictionary = new DictionaryHandler(DictionaryLanguages.ENtoDE, DictionaryWebServices.GOOGLE_TRANSLATE, this);
				myPronunciation = new TTSHandler(TTSLanguages.EN, context);
				setLocale("en");
			}else if(currentBook.getLanguage().equals("de")){
				myDictionary = new DictionaryHandler(DictionaryLanguages.DEtoEN, DictionaryWebServices.GOOGLE_TRANSLATE, this);
				myPronunciation = new TTSHandler(TTSLanguages.DE, context);
				setLocale("de");
			}else{
				myDictionary = new DictionaryHandler(DictionaryLanguages.ENtoDE, DictionaryWebServices.GOOGLE_TRANSLATE, this);
				myPronunciation = new TTSHandler(TTSLanguages.DEFAULT, context);
			}
		}else{
			myDictionary = new DictionaryHandler(DictionaryLanguages.ENtoDE, DictionaryWebServices.GOOGLE_TRANSLATE, this);
			myPronunciation = new TTSHandler(TTSLanguages.DEFAULT, context);
		}
		
		
		//prepare view settings
		this.closeButton = closeButton;
		this.learningView = textswitcher; 
		this.learningView.setFactory(new ViewFactory() {
			
			@Override
			public View makeView() {
				// TODO Auto-generated method stub
				TextView learningTxt = new TextView(context);
				learningTxt.setGravity(Gravity.LEFT);
				FrameLayout.LayoutParams params = new FrameLayout.LayoutParams(
			    LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT,Gravity.LEFT);
				learningTxt.setLayoutParams(params);
				learningTxt.setPadding(10, 10, 35, 10);
				learningTxt.setTextSize(16);
				learningTxt.setLineSpacing(1f, 1.5f);
				learningTxt.setTextColor(Color.WHITE);
				learningTxt.setMaxLines(250);
				learningTxt.setVerticalScrollBarEnabled(true);
				learningTxt.setMovementMethod(new ScrollingMovementMethod());
				return learningTxt;
			}
		});
		this.learningView.setInAnimation(AnimationUtils.loadAnimation(context, R.anim.slide_down));
		this.learningView.setOutAnimation(AnimationUtils.loadAnimation(context, R.anim.slide_up));

	}

	public void learnText(CharSequence txt, LearningServices type){
		 switch (type) {
         case TRANSLATE:
         	 this.learningService = LearningServices.TRANSLATE;
        	 break; 
         case SYNONYMS:
         	 this.learningService = LearningServices.SYNONYMS;
        	 break; 
         case GRAMMAR:
         	 this.learningService = LearningServices.GRAMMAR;
        	 break; 
         case EXPLANATION:
        	 this.learningService = LearningServices.EXPLANATION;
        	 break; 
         case EXAMPLES:
        	 this.learningService = LearningServices.EXAMPLES;
        	 break; 	 
         default:
         	break;
		 }
		 
		myDictionary.lookupWord(txt.toString());
		this.word = txt.toString();
	}
	
	private void showLayer(){
		this.closeButton.setVisibility(View.VISIBLE);
		this.learningView.setVisibility(View.VISIBLE);
	}
	
	public void closeLayer(){
		this.learningView.setVisibility(View.INVISIBLE);
		this.closeButton.setVisibility(View.INVISIBLE);
	}

	public void pronounce (CharSequence txt){
		
    	myPronunciation.startSpeaking(txt.toString());
	}
	
	public void setLearningLanguage(DictionaryLanguages dictLang){
		this.myDictionary.setLanguage(dictLang);
		
		if(dictLang == DictionaryLanguages.DEtoEN){
			this.myPronunciation.setTextLang(TTSLanguages.DE);
			setLocale("de");
		}else if(dictLang == DictionaryLanguages.ENtoDE){
			this.myPronunciation.setTextLang(TTSLanguages.EN);
			setLocale("en");
		}
		
	}
	
	public void setLearningWebService(DictionaryWebServices dictWS){
	
		this.myDictionary.setWebservice(dictWS);
		
	}
	
	public DictionaryWebServices getLearningWebService(){
		
		return this.myDictionary.getWebservice();
		
	}
	
	public DictionaryLanguages getLearningLanguage(){
		
		return this.myDictionary.getLanguage();
		
	}
	
	@SuppressLint("DefaultLocale")
	@SuppressWarnings("rawtypes")
	@Override
	public void onDictionaryCompleted(DictionaryEntry result) {
		// TODO Auto-generated method stub
		
		switch (this.learningService) {
        case TRANSLATE:
         
       	 String formattedTxt = "";
       	 if(result.getTranslation() != null){
       		formattedTxt += "\n"+ result.getTranslation(); 
       	 }else{
    	 		formattedTxt += "\n" + context.getResources().getString(R.string.layertranserror);
    	 }
    	 
       	 
       	 showFormattedText(context.getResources().getString(R.string.layertrans) + "\""+ this.word +  "\":", formattedTxt);
       	 break; 
        case SYNONYMS:
        	HashMap<String, String[]> allSynonyms = result.getSynonyms();
       	 	String formattedSynonyms = "";
       	 	if(allSynonyms != null){
       	 		if(!allSynonyms.isEmpty()){
	       	 		Iterator<Entry<String, String[]>> it = allSynonyms.entrySet().iterator();
					while (it.hasNext()) {
					       
							Map.Entry pairs = (Map.Entry)it.next();
					        String type =  new String(pairs.getKey().toString()).toUpperCase();
					        String[] synonyms = (String[]) pairs.getValue();
					        String synText = "";
					        for (String syn : synonyms) {
								synText += syn + ", ";
							}
					        formattedSynonyms += "\n" + type + ":\n " + synText;
					        it.remove(); 
					}
       	 		}
	       	 	
       	 	}else{
       	 		formattedSynonyms += "\n" + context.getResources().getString(R.string.layersynerror);
       	 	}
       	 
       	 	showFormattedText(context.getResources().getString(R.string.layersyn) + "\""+ this.word +  "\":", formattedSynonyms);
       	 	break; 
         case GRAMMAR:
        	 HashMap<String, String[]> allGrammarInfos = result.getGrammar();
        	 	String formattedGrammarInfos = "";
        	 	if(allGrammarInfos != null){
        	 		if(!allGrammarInfos.isEmpty()){
 	       	 		Iterator<Entry<String, String[]>> it = allGrammarInfos.entrySet().iterator();
 					while (it.hasNext()) {
 					        Map.Entry pairs = (Map.Entry)it.next();
 					        String type =  new String(pairs.getKey().toString()).toUpperCase();
 					        String[] grammar = (String[]) pairs.getValue();
 					        String grText = "";
 					        for (String gr : grammar) {
 								grText += gr + ", ";
 							}
 					        formattedGrammarInfos += "\n" + type + ":\n " + grText;
 					        it.remove(); 
 					}
        	 		}
 	       	 	
        	 	}else{
        	 		formattedGrammarInfos += "\n" + context.getResources().getString(R.string.layergrammarerror);
        	 	}
        	 
        	 	showFormattedText(context.getResources().getString(R.string.layergrammar) + "\""+ this.word +  "\":", formattedGrammarInfos);
        	 	break;  
         case EXPLANATION:
        	HashMap<String, String> allExplanations = result.getExplanation();
     	 	String formattedExpl = "";
     	 	if(allExplanations != null){
     	 		if(!allExplanations.isEmpty()){
	       	 		Iterator it = allExplanations.entrySet().iterator();
					while (it.hasNext()) {
					        Map.Entry pairs = (Map.Entry)it.next();
					        String type =  new String(pairs.getKey().toString()).toUpperCase();
					        String explText = (String) pairs.getValue();
					       
					        formattedExpl += "\n" + type + ":\n " + explText;
					        it.remove(); 
					}
     	 		}
	       	 	
     	 	}else{
     	 		formattedExpl += "\n" + context.getResources().getString(R.string.layerexplerror);
     	 	}
     	 
     	 	showFormattedText(context.getResources().getString(R.string.layerexpl) + "\""+ this.word +  "\":", formattedExpl);
        	
        	
        	 break; 	 	
         case EXAMPLES:
        	
        	 String[] allExamples = result.getExamples();
        	 String formattedExamples = "";
        	 if(allExamples != null){
        		for (int i=0; i < allExamples.length; i++) {
					formattedExamples += "\n"+ Html.fromHtml(allExamples[i]) + "\n";
					if(i > 5){
						break;
					}
				} 
     			
     		 }else{
        	 		formattedExamples += "\n" + context.getResources().getString(R.string.layeregerror);
        	 }
        	 
        	 showFormattedText(context.getResources().getString(R.string.layereg) + "\""+ this.word +  "\":", formattedExamples);
        	
        	 break; 		
        default:
        	 showFormattedText(context.getResources().getString(R.string.layererrorhead),context.getResources().getString(R.string.layererror));
        	break;
		 }
		
		
		showLayer();
	}
	
	private void setLocale(String lang) { 
	    Locale myLocale = new Locale(lang); 
	    Resources res = context.getResources(); 
	    DisplayMetrics dm = res.getDisplayMetrics(); 
	    Configuration conf = res.getConfiguration(); 
	    conf.locale = myLocale; 
	    res.updateConfiguration(conf, dm); 
	  
	} 
	
	private void showFormattedText(String headline, String formattedTxt){
	 
		 SpannableStringBuilder sb = new SpannableStringBuilder();
		 sb.append(headline + formattedTxt);
	   	 sb.setSpan(new TextAppearanceSpan(this.context,R.style.headline), 0, headline.length(), 0);
	     TextView currentText = (TextView) this.learningView.getNextView();
	     currentText.setText(sb, BufferType.SPANNABLE);
	     this.learningView.showNext();
	     
	}
}
