/********************************************************
 * 	eBOOKLEARNING - 
 *	E-book-based interactive learning system
 * ======================================================
 *
 * License: http://www.gnu.org/licenses/gpl.html
 * @author Florence Adegeye
 * @version: 1.0, 2015/01/31
 *
 ********************************************************/
package at.tuwien.ebooklearning.views;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;
import android.app.Activity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import at.tuwien.ebooklearning.R;
import at.tuwien.ebooklearning.ebookservices.BookInfo;
import at.tuwien.ebooklearning.ebookservices.BooksManager;
import at.tuwien.ebooklearning.utils.eBookLearningConstants;


public class BookShelfActivity extends Activity {
	
    private ArrayList<String> allbookinfos = new ArrayList<String>();
	private ArrayAdapter<String> mybooksadapter;
	private ListView booklist;
	private TextView nobooks;
	private final int CHOOSE_FILE = 1;
	private int bookCount = 0;
	private static FileInputStream fileInputStream;
	private static FileOutputStream fileOutputStream;
	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_book_shelf);
		
		bookCount = BooksManager.loadbooks().size();
		
		booklist = (ListView)findViewById(R.id.listbooksView);
		nobooks = (TextView) findViewById(R.id.textnobook);
		mybooksadapter = new ArrayAdapter<String>(this,android.R.layout.simple_list_item_1, allbookinfos);

		booklist.setAdapter(mybooksadapter);
		booklist.setOnItemClickListener(new OnItemClickListener() {

			@Override
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				
			  	//open book
			    BookInfo currentBook = BooksManager.loadbooks().get(position);
			    BooksManager.setCurrentBook(currentBook);
			    if(currentBook != null){
			    	 Log.i("bookinfo", "booki " + currentBook.getTitle() + " lang: " + currentBook.getLanguage());
					    
					 Intent tocIntent = new Intent(getApplicationContext(), BookToCActivity.class);    
					 startActivity(tocIntent);
			    }
			   
			   
	
			}
		});
		
		boolean loaded = (Boolean) getIntent().getExtras().get("loaded");
		
		if(!loaded){
			booklist.setVisibility(View.INVISIBLE);
			nobooks.setVisibility(View.VISIBLE);
		}else{
			booklist.setVisibility(View.VISIBLE);
			nobooks.setVisibility(View.INVISIBLE);
			

			createBookList(BooksManager.loadbooks());
			
			
		}

		
	}

	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.book_shelf, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		   if(item.getItemId() == R.id.action_book){
			   Intent fileSelect;
				Intent intentChooser;
		
				fileSelect = new Intent(Intent.ACTION_GET_CONTENT);
				fileSelect.setType("file/*");
		
				intentChooser = Intent.createChooser(fileSelect, "Choose an .epub file");

				startActivityForResult(intentChooser, CHOOSE_FILE); 
				
				return true;
			
		   }
		   
		 
		   return false;	
			
		
	}
	
	
	public static boolean copyFile(File src, File dst) {
		 
		 
		try {
			FileChannel inChannel;
			fileInputStream = new FileInputStream(src);
			inChannel = fileInputStream.getChannel();
			
			fileOutputStream = new FileOutputStream(dst);
			FileChannel outChannel = fileOutputStream.getChannel();
			inChannel.transferTo(0, inChannel.size(), outChannel);
			
		    inChannel.close();
		    outChannel.close();  
		    
		    return true;
		    
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		} catch (IOException ei) {
			// TODO Auto-generated catch block
			ei.printStackTrace();
			return false;
		}
	  
    } 
	
	/* (non-Javadoc)
	 * @see android.app.Activity#onActivityResult(int, int, android.content.Intent)
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		
		switch (requestCode) {
			case CHOOSE_FILE:
				if(resultCode == RESULT_OK){
					Uri uri = data.getData();
					String filePath = uri.getPath();
					
					Log.i(BookShelfActivity.class.getName(),filePath);
					 File src = new File(filePath);
					 File filedir = getApplicationContext().getFilesDir();
					
					 String myBooksDir = filedir+File.separator+eBookLearningConstants.DEFAULT_DIR+"_"+bookCount;
					 File bookdir = new File(myBooksDir);
					 if(!bookdir.mkdir()){
						 bookdir.mkdir();
					 }
				
					 File dst = new File(bookdir,src.getName());
				   
					boolean copySuccess = copyFile(src, dst);
					
					if(copySuccess){
						bookCount++;
						
					    updateEPubs(dst.getAbsolutePath(), eBookLearningConstants.EBOOK_FILE_EXT);
						Log.i("bookshelfactivity","mybooks: " + BooksManager.loadbooks() + " amount " + BooksManager.loadbooks().size());
					}
					
					if(allbookinfos.size() == 1){
						booklist.setVisibility(View.VISIBLE);
						nobooks.setVisibility(View.INVISIBLE);
					}
				}

		
		}
		
		
	}


	private void createBookList(HashMap<Integer, BookInfo> myBooks){
	    
		Iterator<Entry<Integer, BookInfo>> itBook =  myBooks.entrySet().iterator();
		
		while(itBook.hasNext()){
			 BookInfo bookinfo = itBook.next().getValue();
			 allbookinfos.add(bookinfo.getShortInfo());
			 
		}
		
	}
	
	private boolean updateEPubs(String path, String extension){
		File newfile = new File(path);
		int bookID = BooksManager.loadbooks().size();
			
		if(newfile.isFile() && newfile.getName().endsWith(extension) && mybooksadapter != null){
			BookInfo currentBookInfo = BooksManager.createBookInfo(newfile);
			currentBookInfo.setBookid(bookID);
			BooksManager.loadbooks().put(bookID, currentBookInfo);	
			allbookinfos.add(currentBookInfo.getShortInfo());
			mybooksadapter.notifyDataSetChanged();
				
			
			return true;
		}else{
			Log.e("loadEPub", "No valid file");
			Toast.makeText(getApplicationContext(), "Bookloading error - Invalid file", Toast.LENGTH_SHORT).show();
			return false;
		}
		
		
	}
	
	

	/* (non-Javadoc)
	 * @see android.app.Activity#onDestroy()
	 */
	@Override
	protected void onDestroy() {
		// TODO Auto-generated method stub
		BooksManager.loadbooks().clear();
		super.onDestroy();
	}


	
}
