/********************************************************
 * 	eBOOKLEARNING - 
 *	E-book-based interactive learning system
 * ======================================================
 *
 * License: http://www.gnu.org/licenses/gpl.html
 * @author Florence Adegeye
 * @version: 1.0, 2015/01/31
 *
 ********************************************************/
package at.tuwien.ebooklearning.views;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.net.wifi.p2p.WifiP2pManager.ActionListener;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Html;
import android.text.Spanned;
import android.util.Log;
import android.view.ActionMode;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnLongClickListener;
import android.widget.Button;
import android.widget.TextSwitcher;
import android.widget.TextView;
import android.widget.Toast;
import at.tuwien.ebooklearning.R;
import at.tuwien.ebooklearning.dictionary.DictionaryLanguages;
import at.tuwien.ebooklearning.dictionary.DictionaryWebServices;
import at.tuwien.ebooklearning.ebookservices.BookImageLoader;
import at.tuwien.ebooklearning.ebookservices.BookInfo;
import at.tuwien.ebooklearning.ebookservices.BooksManager;
import at.tuwien.ebooklearning.utils.LearningManager;
import at.tuwien.ebooklearning.utils.LearningServices;




public class ContentActivity extends Activity{
	
	TextView bookContent;
	private ActionMode mActionMode = null;
	private BookInfo currentBook;
	private LearningManager myLearningTools;
    private  AlertDialog.Builder builderTools = null;
    private  AlertDialog.Builder builderService = null;
    private  AlertDialog.Builder builderLang = null;
    private CharSequence textSelected;
    private int checkedService = -1;
    private int checkedLang = -1;

    
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_content);
	
		//create text
		
		bookContent = (TextView) findViewById(R.id.bookContent);
		bookContent.setTextIsSelectable(true); 
	
	    currentBook= BooksManager.getCurrentBook();
	    String displayString = getIntent().getExtras().getString("chapter");
	    myLearningTools = new LearningManager(getApplicationContext(), (TextSwitcher) findViewById(R.id.textSwitcher1), (Button) findViewById(R.id.closelayer), currentBook);
	    
	    if(myLearningTools.getLearningLanguage() == DictionaryLanguages.DEtoEN){
			checkedLang = 1;
			
		}else if(myLearningTools.getLearningLanguage() == DictionaryLanguages.ENtoDE){
			
			checkedLang = 0;
		}
		
	    
		if(myLearningTools.getLearningWebService() == DictionaryWebServices.GOOGLE_TRANSLATE){
			
			checkedService= 1;
		
		}else if(myLearningTools.getLearningWebService() == DictionaryWebServices.GLOSBE){
			
			checkedService = 0;
		}
	    
	    
	    //create optionDialogs
	
	    builderTools = new AlertDialog.Builder(this);
	    builderTools.setNegativeButton(getResources().getString(R.string.cancel), new DialogInterface.OnClickListener(){

			@Override
			public void onClick(DialogInterface dialog, int which) {
				// TODO Auto-generated method stub
				dialog.cancel();
				
			}
	    	
	    });
	  
	    builderService = new AlertDialog.Builder(this);
	    builderService.setNegativeButton("Ok", new DialogInterface.OnClickListener(){

			@Override
			public void onClick(DialogInterface dialog, int which) {
				// TODO Auto-generated method stub
				dialog.cancel();
			}
	    	
	    }); 
	    
	   
	    builderLang = new AlertDialog.Builder(this);
	    builderLang.setNegativeButton("Ok", new DialogInterface.OnClickListener(){

			@Override
			public void onClick(DialogInterface dialog, int which) {
				// TODO Auto-generated method stub
				if(mActionMode != null){
					mActionMode.finish();
				}
				
				dialog.cancel();
			}
	    	
	    });
	    
	    
	    if(displayString != null){
	    	new LoadImage().execute(displayString);
	    	
	    }	
	  
	  
	}
    
	public void onActionItemClicked(MenuItem item){
		
		//get text selection
    	int min = 0;
        int max = bookContent.getText().length();
        if (bookContent.isFocused()) {
            final int selStart = bookContent.getSelectionStart();
            final int selEnd = bookContent.getSelectionEnd();

            min = Math.max(0, Math.min(selStart, selEnd));
            max = Math.max(0, Math.max(selStart, selEnd));
        }
       textSelected = bookContent.getText().subSequence(min, max);
		
	    switch (item.getItemId()) {
	    	case R.id.learningitem:
	    		
	    		Log.i("option","learning");
	    		
	    	    CharSequence tools[] = new CharSequence[] {getResources().getString(R.string.translate),getResources().getString(R.string.grammar),getResources().getString(R.string.synonym),
	    	    		getResources().getString(R.string.expl), getResources().getString(R.string.eg), getResources().getString(R.string.tts)};
	    	    builderTools.setTitle(getResources().getString(R.string.tooltxt));
	    		builderTools.setItems(tools, new DialogInterface.OnClickListener() {
	    	        @Override
	    	        public void onClick(DialogInterface dialog, int which) {
	    	        	switch (which) {
	    	        		case 0:
	    	        			onContextualMenuItemClicked(0);
	    	        			break;
	    	        		case 1:
	    	        			onContextualMenuItemClicked(1);
	    	        			break;
	    	        		case 2:
	    	        			onContextualMenuItemClicked(2);
	    	        			break;
	    	        		case 3:
	    	        			onContextualMenuItemClicked(3);
	    	        			break;
	    	        		case 4:
	    	        			onContextualMenuItemClicked(4);
	    	        			break;
	    	        		case 5:
	    	        			onContextualMenuItemClicked(5);
	    	        			break;
	    	        		default:
	    	        			dialog.cancel();
	    	        			break;
	    	           }
	    	        }
	    	    }).create();
	    		
	    		builderTools.show();
	    		break;
	    	case R.id.settingService:
	    		
	    		Log.i("option","services");
	    		  
	    	    CharSequence services[] = new CharSequence[] {"Glosbe.com", "Goggle Translate"};
	    	    builderService.setTitle(getResources().getString(R.string.servicetxt)); 
	    		builderService.setSingleChoiceItems(services, checkedService, new DialogInterface.OnClickListener() {
	    			
	    			@Override
	    			public void onClick(DialogInterface dialog, int which) {
	    				// TODO Auto-generated method stub
	    				switch (which) {
	    					case 0:
	    						onSettingMenuItemClicked(0);
	    						break;
	    					case 1:
	    						onSettingMenuItemClicked(1);
	    	        			break;
	    	        		default:
	    	        			break;
	    				}
	    			}
	    		}).create();
	    		
	    		builderService.show();
	    		break;
	    	case R.id.settingLang:
	
				Log.i("option","lang");
				 
			    CharSequence langs[] = new CharSequence[] {getResources().getString(R.string.eng), getResources().getString(R.string.ger)};
			    builderLang.setTitle(getResources().getString(R.string.langtxt));
				  builderLang.setSingleChoiceItems(langs, checkedLang, new DialogInterface.OnClickListener() {
						
						@Override
						public void onClick(DialogInterface dialog, int which) {
							// TODO Auto-generated method stub
							switch (which) {
								case 0:
									onSettingMenuItemClicked(2);
									
									break;
								case 1:
									onSettingMenuItemClicked(3);
									break;
				        		default:
				        			break;
							}
						}
					}).create();
				
				builderLang.show();
				break;
	    	default:
	    		break;
	    }
       
      
	}

	public void onContextualMenuItemClicked(int item) {
    	
	    
        final CharSequence selectedText =  textSelected; 
      
        
        switch (item) {
            case 0:
            	
            	myLearningTools.learnText(selectedText, LearningServices.TRANSLATE);    	 
            	Log.i("actionitem", "translate: "+ selectedText);
            	break; 
            	  
            case 5:
            	myLearningTools.pronounce(selectedText);
            	Log.i("actionitem", "tts");
                break; 	  
           	case 2:
           		myLearningTools.learnText(selectedText, LearningServices.SYNONYMS);   
            	Log.i("actionitem", "synonym");
                break;
            case 1:
            	myLearningTools.learnText(selectedText, LearningServices.GRAMMAR);   
            	Log.i("actionitem", "grammar");
                break;
            case 3:
            	myLearningTools.learnText(selectedText, LearningServices.EXPLANATION);   
            	Log.i("actionitem", "explanation");
                break;    
            case 4:
            	myLearningTools.learnText(selectedText, LearningServices.EXAMPLES);   
            	Log.i("actionitem", "examples");
                break;
           
            default:
            	break;
          
           
        }
 
      
    }

	
	public void onSettingMenuItemClicked(int item) {
		   
		   
		   if(item == 3){
			   myLearningTools.setLearningLanguage(DictionaryLanguages.DEtoEN);
			   checkedLang = 1;
			   Toast.makeText(this.getApplicationContext(),getResources().getString(R.string.transgereng), Toast.LENGTH_SHORT).show();
			   Log.i(BookShelfActivity.class.getName(),"Default-Translation from German to English");
			 
			   
		   }else if(item  == 2){
			   myLearningTools.setLearningLanguage(DictionaryLanguages.ENtoDE);
			   checkedLang = 0;
			   Toast.makeText(this.getApplicationContext(),getResources().getString(R.string.transengger), Toast.LENGTH_SHORT).show();
			   Log.i(BookShelfActivity.class.getName(),"Default-Translation from English to German");
			  
		   }else if(item == 0){
			   myLearningTools.setLearningWebService(DictionaryWebServices.GLOSBE);
			   checkedService = 0;
			   Toast.makeText(this.getApplicationContext(),getResources().getString(R.string.usedservice)+" Glosbe.com", Toast.LENGTH_SHORT).show();
			   Log.i(BookShelfActivity.class.getName(),"Used Translation-Service: Glosbe.com");
			  
		   }else if(item == 1){
			   myLearningTools.setLearningWebService(DictionaryWebServices.GOOGLE_TRANSLATE);
			   checkedService = 1;
			   Toast.makeText(this.getApplicationContext(),getResources().getString(R.string.usedservice)+" Google translate", Toast.LENGTH_SHORT).show();
			   Log.i(BookShelfActivity.class.getName(),"Used Translation-Service: Google translate");
			  
		   }
		  
			
		
	}

	
	

	/* (non-Javadoc)
	 * @see android.app.Activity#onActionModeStarted(android.view.ActionMode)
	 */
	@Override
	public void onActionModeStarted(ActionMode mode) {
		// TODO Auto-generated method stub
		invalidateOptionsMenu();
		if (mActionMode == null) {
            mActionMode = mode;
            Menu menu = mode.getMenu();
            // Remove the default menu items (select all, copy, paste, search etc)
            menu.clear();
          
            mode.getMenuInflater().inflate(R.menu.content, menu);
           
          
    		
        }

      
      
	}

	/* (non-Javadoc)
	 * @see android.app.Activity#onActionModeFinished(android.view.ActionMode)
	*/ 
	@Override
	public void onActionModeFinished(ActionMode mode) {
		// TODO Auto-generated method stub

		
		mActionMode = null;
		super.onActionModeFinished(mode);
	}
	
	public void onHomePressed(View view){
		this.finish();
		Intent tocIntent = new Intent(getApplicationContext(), BookToCActivity.class);    
		startActivity(tocIntent);
	}
	

	public void onLearningLayerClosed(View view){
		myLearningTools.closeLayer();
		
	}
	
	
	class LoadImage extends AsyncTask<Object,Void,Spanned> {

		@Override
		protected Spanned doInBackground(Object... params) {
			// TODO Auto-generated method stub
			String displayString= (String) params[0];
			
			Spanned spanned = Html.fromHtml(displayString, new BookImageLoader(getApplicationContext(), currentBook), null);
			
			return spanned;
		}

		/* (non-Javadoc)
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Spanned result) {
			// TODO Auto-generated method stub
			super.onPostExecute(result);
			bookContent.setText(result);
		}
		
		
		
	}



}
