/********************************************************
 * 	eBOOKLEARNING - 
 *	E-book-based interactive learning system
 * ======================================================
 *
 * License: http://www.gnu.org/licenses/gpl.html
 * @author Florence Adegeye
 * @version: 1.0, 2015/01/31
 *
 ********************************************************/
package at.tuwien.ebooklearning.webservices;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.os.AsyncTask;
import android.util.Log;
import at.tuwien.ebooklearning.dictionary.DictionaryEntry;
import at.tuwien.ebooklearning.utils.eBookLearningConstants;

public class GlosbeJSONService extends AsyncTask<String, Void, String> implements IWebDictionaryService{
	
	private IDictionaryResult dictResult;
	
	public GlosbeJSONService(IDictionaryResult dictResult) {
		super();
		this.dictResult = dictResult;
	}
	
	
	@Override
	protected String doInBackground(String... inputs) {
		// TODO Auto-generated method stub
		
		String glosbeResult = requestDictionary(inputs[0], inputs[1], inputs[2]);
		return glosbeResult;
	}

	/* (non-Javadoc)
	 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
	 */
	@Override
	protected void onPostExecute(String result) {
		
		if(result != null){
			DictionaryEntry dictEntry = createDictionaryEntry(result);
			dictResult.onDictionaryCompleted(dictEntry);
		}else{
			Log.e(GlosbeJSONService.class.getName(), "Failed to connect to glosbe.com");
		}
	
		
	}
	 
	@Override
	public String requestDictionary(String from, String dest, String phrase) {
		 String specialChar = phrase.replaceAll("(\\W)(\\s)", " ");
		  
		   String formattedURL;
		   String glosbeURL;
		   StringBuilder builder = new StringBuilder();
		   HttpClient client = new DefaultHttpClient();
			try {
				formattedURL = URLEncoder.encode(specialChar, "UTF-8");
				
				glosbeURL = eBookLearningConstants.GLOSBE1_FROM+from+eBookLearningConstants.GLOSBE2_DEST+dest+eBookLearningConstants.GLOSBE3_PHRASE+ formattedURL;
				Log.i(GlosbeJSONService.class.getName(),"Connect to: " + glosbeURL);
			    HttpGet httpGet = new HttpGet(glosbeURL);
			    
			    HttpResponse response = client.execute(httpGet);
			      StatusLine statusLine = response.getStatusLine();
			      int statusCode = statusLine.getStatusCode();
			      if (statusCode == 200) {
			        HttpEntity entity = response.getEntity();
			        InputStream content = entity.getContent();
			        BufferedReader reader = new BufferedReader(new InputStreamReader(content));
			        String line;
			        while ((line = reader.readLine()) != null) {
			          builder.append(line);
			         
			        }
			        return builder.toString();
			      } else {
			        Log.e(GlosbeJSONService.class.getName(), "Failed to connect to glosbe.com");
			      }
				
				
			} catch (UnsupportedEncodingException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (ClientProtocolException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		  
		   
		    return null;
	}

	@Override
	public DictionaryEntry createDictionaryEntry(String input) {
		// TODO Auto-generated method stub
		//translation
		DictionaryEntry myWord = new DictionaryEntry();
		String translation = "";
		JSONArray tuc = null;
		//Log.i("sdf", input);
		try {
			
			JSONObject glosbeResponse = new JSONObject(input);
			if(!glosbeResponse.isNull(eBookLearningConstants.G_TUC)){
				tuc = glosbeResponse.getJSONArray(eBookLearningConstants.G_TUC);
		        if (!tuc.isNull(0)) {
					JSONObject translationObj = tuc.getJSONObject(0);
					if (!translationObj.isNull(eBookLearningConstants.G_PHRASE)) {

						JSONObject phrase = translationObj
								.getJSONObject(eBookLearningConstants.G_PHRASE);

						if (!phrase.isNull(eBookLearningConstants.G_TEXT)) {

							translation = phrase
									.getString(eBookLearningConstants.G_TEXT);
							myWord.setTranslation(translation);
						}
					}
				}else{
					 Log.e(GlosbeJSONService.class.getName(), "No translation found");
				}
			}else{
				 Log.e(GlosbeJSONService.class.getName(), "No translation found");
			}
			
	         
	        //synonyms and description (explanations)
	        HashMap<String, String> allSynDescription = new HashMap<String, String>();
	        for(int i = 0; i < tuc.length();i++){
	        	if(i <= 5){
	        		JSONObject allObj = tuc.getJSONObject(i);
	        		String currentMeaning = " ";
	        		String currentSynonym = " "; 
	        		
	        		if(!allObj.isNull(eBookLearningConstants.G_PHRASE)){
	        			 JSONObject currentPhrase = allObj.getJSONObject(eBookLearningConstants.G_PHRASE);
	        			 if(!currentPhrase.isNull(eBookLearningConstants.G_TEXT)){
	        				 currentSynonym = currentPhrase.getString(eBookLearningConstants.G_TEXT);
	        			 }
	        		}
	        		
	        		if(!allObj.isNull(eBookLearningConstants.G_MEANINGS)){
	        			 JSONArray currentMeaningArray = allObj.getJSONArray(eBookLearningConstants.G_MEANINGS);
	        			 if(!currentMeaningArray.getJSONObject(0).isNull(eBookLearningConstants.G_TEXT)){
	        				 currentMeaning = currentMeaningArray.getJSONObject(0).getString(eBookLearningConstants.G_TEXT);
	        			 }
	        		}
			        allSynDescription.put(currentSynonym, currentMeaning);
	        	}else{
	        		break;
	        	}
	        	
	        }
	        myWord.setExplanation(allSynDescription);
	        
	        //examples
	        String[] examples = new String[6];
	        if(!glosbeResponse.isNull(eBookLearningConstants.G_EXAMPLES)){
	        	 JSONArray examplesObj = glosbeResponse.getJSONArray(eBookLearningConstants.G_EXAMPLES);
	 	        for(int i = 0; i < examplesObj.length();i++){
	 	        	if(i <= 5){
	 	        		JSONObject allExamples = examplesObj.getJSONObject(i);
	 	        		String firstExample = " ";
	 	        		if(!allExamples.isNull(eBookLearningConstants.G_FIRST)){
	 	        			firstExample = allExamples.getString(eBookLearningConstants.G_FIRST);
	 	        		}
	 			       
	 			        examples[i] = firstExample;
	 	        	}else{
	 	        		break;
	 	        	}
	 	        	
	 	        }
	 	       myWord.setExamples(examples);
	        }else{
	        	 Log.e(GlosbeJSONService.class.getName(), "No examples found");
	        }
	       
	      
			
			
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return myWord;
	}

	
}
