/********************************************************
 * 	eBOOKLEARNING - 
 *	E-book-based interactive learning system
 * ======================================================
 *
 * License: http://www.gnu.org/licenses/gpl.html
 * @author Florence Adegeye
 * @version: 1.0, 2015/01/31
 *
 ********************************************************/
package at.tuwien.ebooklearning.webservices;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;

import android.os.AsyncTask;
import android.util.Log;
import at.tuwien.ebooklearning.dictionary.DictionaryEntry;
import at.tuwien.ebooklearning.utils.eBookLearningConstants;

public class GoogleJSONService extends AsyncTask<String, Void, String> implements IWebDictionaryService{
	
	private IDictionaryResult dictResult;
	
	public GoogleJSONService(IDictionaryResult dictResult) {
		super();
		this.dictResult = dictResult;
	}

	@Override
	protected String doInBackground(String... inputs) {
		// TODO Auto-generated method stub
		
		String googleResult = requestDictionary(inputs[0], inputs[1], inputs[2]);
		return googleResult;
	}

	/* (non-Javadoc)
	 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
	 */
	@Override
	protected void onPostExecute(String result) {
		// TODO Auto-generated method stub
		if(result != null){
			DictionaryEntry dictEntry= createDictionaryEntry(result);
			dictResult.onDictionaryCompleted(dictEntry);
		}else{
			Log.e(GoogleJSONService.class.getName(), "Failed to connect to translate.google.com");
		}
		
	}

	@Override
	public String requestDictionary(String from, String dest, String phrase) {
		// TODO Auto-generated method stub
		   String specialChar = phrase.replaceAll("(\\W)(\\s)", " ");
		  
		   String formattedURL;
		   String goggleURL;
		   StringBuilder builder = new StringBuilder();
		   HttpClient client = new DefaultHttpClient();
			try {
				formattedURL = URLEncoder.encode(specialChar, "UTF-8");
				
				goggleURL = eBookLearningConstants.GOOGLE1_FROM+from+eBookLearningConstants.GOOGLE2_DEST+dest+eBookLearningConstants.GOOGLE3_PHRASE+formattedURL;
				Log.i(GoogleJSONService.class.getName(),"Connect to: " + goggleURL);
			    HttpGet httpGet = new HttpGet(goggleURL);
			    
			    HttpResponse response = client.execute(httpGet);
			      StatusLine statusLine = response.getStatusLine();
			      int statusCode = statusLine.getStatusCode();
			      if (statusCode == 200) {
			        HttpEntity entity = response.getEntity();
			        InputStream content = entity.getContent();
			        BufferedReader reader = new BufferedReader(new InputStreamReader(content));
			        String line;
			        while ((line = reader.readLine()) != null) {
			          builder.append(line);
			          
			        }
			        return builder.toString();
			        
			        
			      } else {
			        Log.e(GoogleJSONService.class.getName(), "Failed to connect to translate.google.com");
			      }
				
				
			} catch (UnsupportedEncodingException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (ClientProtocolException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		  
		   
		    return null;
	}

	@Override
	public DictionaryEntry createDictionaryEntry(String input) {
		DictionaryEntry myWord = new DictionaryEntry();
		//Log.i("sdf", input);
		try {
			
			JSONArray googleResponse = new JSONArray(input);
			
			//translation values
			String wordTranslated = "";
			if(!googleResponse.isNull(0)){
				JSONArray arrayTranslation = googleResponse.getJSONArray(0); 
				if(!arrayTranslation.isNull(0)){
					JSONArray translationValue = arrayTranslation.getJSONArray(0);
			        wordTranslated = translationValue.getString(0);
			        myWord.setTranslation(wordTranslated);
			         
				}
		      
			}else{
				Log.e(GoogleJSONService.class.getName(), "No translation found");
			}
	       
	       //grammar values  
	        if (!googleResponse.isNull(1)) {
	        	JSONArray arrayGrammar = googleResponse.getJSONArray(1);
	        	String[] grammar = null;
	        	String wordType="";
	        	HashMap<String, String[]> grammarCollection = new HashMap<String, String[]>();
	        	
	        	for (int i = 0; i < arrayGrammar.length(); i++) {
	        		wordType = arrayGrammar.getJSONArray(i).getString(0);
	        		
	        		JSONArray subValue = arrayGrammar.getJSONArray(i).getJSONArray(1);
	        		grammar = new String[subValue.length()];
					for (int j = 0; j < subValue.length(); j++) {
						grammar[j] = subValue.getString(j);
					}
					grammarCollection.put(wordType, grammar);
	        	}	
	        	
	        	myWord.setGrammar(grammarCollection);
				
			}else{
				Log.e(GoogleJSONService.class.getName(), "No information to grammar");
			}
	        
			//synonyms
	        if (!googleResponse.isNull(11)) {
	        	String[] synonyms = null;
	        	String synonymType="";
	        	HashMap<String, String[]> synonymCollection = new HashMap<String, String[]>();
	        	JSONArray arraySynTypes = googleResponse.getJSONArray(11);
	        	for (int i = 0; i < arraySynTypes.length(); i++) {
	        		synonymType = arraySynTypes.getJSONArray(i).getString(0);
	        		
	        		JSONArray subValue = arraySynTypes.getJSONArray(i).getJSONArray(1).getJSONArray(0).getJSONArray(0);
	        		synonyms = new String[subValue.length()];
					for (int j = 0; j < subValue.length(); j++) {
						synonyms[j] = subValue.getString(j);
					}
					synonymCollection.put(synonymType, synonyms);
	        	}	
	        	myWord.setSynonyms(synonymCollection);
	        		
				
			}else{
				Log.e(GoogleJSONService.class.getName(), "No synonyms");
			}
	       //explanation 
			if (!googleResponse.isNull(12)) {
				
				String subExplanation = "";
	        	String explType="";
	        	HashMap<String, String> explCollection = new HashMap<String, String>();
	        	JSONArray arrayExpl = googleResponse.getJSONArray(12);
	        	for (int i = 0; i < arrayExpl.length(); i++) {
	        		explType= arrayExpl.getJSONArray(i).getString(0);
	        		
	        		JSONArray subValue = arrayExpl.getJSONArray(i).getJSONArray(1).getJSONArray(0);
	        		subExplanation = subValue.getString(0);
	        		
					explCollection.put(explType, subExplanation);
	        	}	
	        	myWord.setExplanation(explCollection);
	        		
				
			}else{
				Log.e(GoogleJSONService.class.getName(), "No explanations");
			}			
			//examples
			if (!googleResponse.isNull(13)) {
				
				JSONArray arrayExamples = googleResponse.getJSONArray(13);
				JSONArray exampl = arrayExamples.getJSONArray(0);
				String[] examples = new String[exampl.length()];
				for (int i = 0; i < exampl.length(); i++) {
					examples[i] = exampl.getJSONArray(i).getString(0);

				}
				myWord.setExamples(examples);
			}else{
				Log.e(GoogleJSONService.class.getName(), "No examples for this word");
			}
			
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			Log.e(GoogleJSONService.class.getName(),"Wrong input - JSON Error");
			e.printStackTrace();
		}
	
		return myWord;
	}

	
}
